@@ -5,14 +5,15 @@ import selectors
 import asyncio
 import httpx
 import nest_asyncio
+import browser_cookie3 as bc
 from dateutil.parser import parse
 from netrc import netrc
 import multiprocessing as mp
 from urllib.parse import urlparse
 from tqdm import tqdm
 
+
 nest_asyncio.apply()
-pro_num = 1
 
 
 def get_url_host(url):
@@ -97,16 +98,15 @@ def _parse_file_name(response):
 
 
 def _unit_formater(size, suffix):
-    if 1024 <= size < 1024 ** 2:
-        return f'{size/1024:.2f}k{suffix}'
-    elif 1024**2 <= size < 1024 ** 3:
-        return f'{size/1024**2:.2f}M{suffix}'
-    elif 1024**3 <= size <= 1024 ** 4:
-        return f'{size/1024**3:.2f}G{suffix}'
-    elif 1024**4 <= size <= 1024 ** 5:
-        return f'{size/1024**3:.2f}T{suffix}'
-    else:
-        return f'{size:.2f}{suffix}'
+    prefixs = ['', 'k', 'M', 'G', 'T']
+    idx = 0
+    while size / 1024 >= 1:
+        size = size / 1024
+        idx += 1
+        if idx == 4:
+            break
+
+    return f'{size:.2f}{prefixs[idx]}{suffix}'
 
 
 def _new_file_from_web(r, file_path):
@@ -120,6 +120,19 @@ def _new_file_from_web(r, file_path):
         return False
 
 
+def _get_cookiejar(authorize_from_browser):
+    cj = None
+    if authorize_from_browser:
+        try:
+            cj = bc.load()
+        except:
+            raise EnvironmentError("Could not load cookie from browser. "
+                                   "Please login in website via browser before run this code"
+                                   "\n  So far the following browsers are supported: "
+                                   "Chrome，Firefox, Opera, Edge, Chromium")
+    return cj
+
+
 def _handle_status(r, url, local_size, file_name, file_path):
     # returns (True, '') : downloaded entirely
     # returns (False,'') : error! break download
@@ -185,7 +198,8 @@ def _handle_status(r, url, local_size, file_name, file_path):
         return False, url_new
     elif r.status_code == 401:
         tqdm.write(
-            '>>> Authorization failed! Please check your username and password in Netrc')
+            '>>> Authorization failed! Please check your username and password in Netrc '
+            'Or authorizing by browser and set the parameter `authorize_from_browser` to `True`')
         return False, ''
     elif r.status_code == 403:
         tqdm.write(
@@ -197,7 +211,8 @@ def _handle_status(r, url, local_size, file_name, file_path):
         return False, ''
 
 
-def download_data(url, folder=None, file_name=None, client=None, retry=0):
+def download_data(url, folder=None, authorize_from_browser=False, file_name=None,
+                  client=None, allow_redirects=False, retry=0):
     '''Download a single file.
 
     Parameters:
@@ -206,13 +221,21 @@ def download_data(url, folder=None, file_name=None, client=None, retry=0):
         url of web file
     folder: str
         the folder to store output files. Default current folder.
+    authorize_from_browser: bool
+        Whether to load cookies used by your web browser for authorization.
+        This means you can use python to download data by logining in to website 
+        via browser (So far the following browsers are supported: Chrome，Firefox, 
+        Opera, Edge, Chromium"). It will be very usefull when website doesn't support
+        "HTTP Basic Auth". Default is False.
     file_name: str
         the file name. If None, will parse from web response or url.
         file_name can be the absolute path if folder is None.
     client: httpx.Client() object
         client maintaining connection. Default None
+    allow_redirects: bool
+        Enables or disables HTTP redirects
     retry: int 
-        the times of reconnetion when status code is 503
+        number of reconnections when status code is 503
     '''
     # init parameters
     global support_resume, pbar, remote_size
@@ -222,7 +245,10 @@ def download_data(url, folder=None, file_name=None, client=None, retry=0):
     if not client:
         client = httpx
 
-    r = client.get(url, headers=headers, timeout=120, allow_redirects=False)
+    cj = _get_cookiejar(authorize_from_browser)
+
+    r = client.get(url, headers=headers, timeout=120,
+                   allow_redirects=allow_redirects, cookies=cj)
     r.close()
 
     if not file_name:
@@ -243,9 +269,11 @@ def download_data(url, folder=None, file_name=None, client=None, retry=0):
         elif status == False:
             if url_new:  # 301,302
                 return download_data(url_new, folder=folder, file_name=file_name,
-                                     client=client, retry=retry - 1)
+                                     authorize_from_browser=authorize_from_browser,
+                                     allow_redirects=True, client=client)
             elif retry > 0:
                 return download_data(url, folder=folder, file_name=file_name,
+                                     authorize_from_browser=authorize_from_browser,
                                      client=client, retry=retry - 1)
             else:  # error! break download
                 return False
@@ -256,7 +284,7 @@ def download_data(url, folder=None, file_name=None, client=None, retry=0):
     else:
         headers = None
 
-    with client.stream("GET", url, headers=headers, timeout=120) as r:
+    with client.stream("GET", url, headers=headers, timeout=120, cookies=cj) as r:
         with open(file_path, "ab") as f:
             time_start_realtime = time_start = time.time()
             for chunk in r.iter_raw():
@@ -287,7 +315,7 @@ def download_data(url, folder=None, file_name=None, client=None, retry=0):
             return True
 
 
-def download_datas(urls, folder=None, file_names=None):
+def download_datas(urls, folder=None, authorize_from_browser=False, file_names=None):
     '''download data from a list like object which containing urls.
     This function will download files one by one.
 
@@ -297,6 +325,12 @@ def download_datas(urls, folder=None, file_names=None):
         iterator contains urls
     folder: str
         the folder to store output files. Default current folder.
+    authorize_from_browser: bool
+        Whether to load cookies used by your web browser for authorization.
+        This means you can use python to download data by logining in to website 
+        via browser (So far the following browsers are supported: Chrome，Firefox, 
+        Opera, Edge, Chromium"). It will be very usefull when website doesn't support
+        "HTTP Basic Auth". Default is False.
     file_names: iterator
         iterator contains names of files. Leaving it None if you want the program to parse
         them from website. file_names can cantain the absolute paths if folder is None.
@@ -321,18 +355,21 @@ def download_datas(urls, folder=None, file_names=None):
     client = httpx.Client(timeout=None)
     for i, url in enumerate(urls):
         if file_names:
-            download_data(url, folder, file_names[i], client)
+            download_data(url, folder, file_name=file_names[i], client=client,
+                          authorize_from_browser=authorize_from_browser)
         else:
-            download_data(url, folder, client=client)
+            download_data(url, folder, client=client,
+                          authorize_from_browser=authorize_from_browser)
 
 
 def _mp_download_data(args):
     return download_data(*args)
 
 
-def mp_download_datas(urls, folder=None, file_names=None, ncore=None, desc=''):
+def mp_download_datas(urls, folder=None,  authorize_from_browser=False, file_names=None,
+                      ncore=None, desc=''):
     '''download data from a list like object which containing urls.
-    This function will download multiple files simultaneously using multiporocess.
+    This function will download multiple files simultaneously using multiprocess.
 
     Parameters:
     -----------
@@ -340,6 +377,12 @@ def mp_download_datas(urls, folder=None, file_names=None, ncore=None, desc=''):
         iterator contains urls
     folder: str
         the folder to store output files. Default current folder.
+    authorize_from_browser: bool
+        Whether to load cookies used by your web browser for authorization.
+        This means you can use python to download data by logining in to website 
+        via browser (So far the following browsers are supported: Chrome，Firefox, 
+        Opera, Edge, Chromium"). It will be very usefull when website doesn't support
+        "HTTP Basic Auth". Default is False.
     file_names: iterator
         iterator contains names of files. Leaving it None if you want the program to parse
         them from website. file_names can cantain the absolute paths if folder is None.
@@ -377,22 +420,27 @@ def mp_download_datas(urls, folder=None, file_names=None, ncore=None, desc=''):
 
     with mp.Pool(ncore) as pool:
         if file_names:
-            args = [(urls[i], folder, file_names[i]) for i in range(len(urls))]
+            args = [(urls[i], folder, authorize_from_browser, file_names[i])
+                    for i in range(len(urls))]
         else:
-            args = [(urls[i], folder) for i in range(len(urls))]
+            args = [(urls[i], folder, authorize_from_browser)
+                    for i in range(len(urls))]
 
         for i in pool.imap_unordered(_mp_download_data, args):
             pbar.update()
 
 
-async def _download_data(client, url, folder=None, file_name=None, retry=0):
+async def _download_data(client, url, folder=None, file_name=None,
+                         authorize_from_browser=False, allow_redirects=False, retry=0):
     global support_resume, pbar, remote_size
 
     headers = {'Range': 'bytes=0-4'}
     support_resume = False
+
+    cj = _get_cookiejar(authorize_from_browser)
     # auth = get_netrc_auth(url)
 
-    r = await client.get(url, headers=headers, timeout=120, allow_redirects=False)
+    r = await client.get(url, headers=headers, timeout=120, allow_redirects=allow_redirects, cookies=cj)
     r.close()
     # r = await client.head(url, headers=headers, auth=auth, timeout=120)
     if not file_name:
@@ -416,9 +464,11 @@ async def _download_data(client, url, folder=None, file_name=None, retry=0):
         elif status == False:
             if url_new:  # 301,302
                 return await _download_data(client, url_new, folder=folder,
-                                            file_name=file_name, retry=retry - 1)
+                                            authorize_from_browser=authorize_from_browser,
+                                            file_name=file_name, allow_redirects=True)
             elif retry > 0:
                 return await _download_data(client, url, folder=folder,
+                                            authorize_from_browser=authorize_from_browser,
                                             file_name=file_name, retry=retry - 1)
             else:  # error! break download
                 return False
@@ -429,7 +479,7 @@ async def _download_data(client, url, folder=None, file_name=None, retry=0):
     else:
         headers = None
     auth = get_netrc_auth(get_url_host(url))
-    async with client.stream('GET', url, headers=headers, auth=auth, timeout=None) as r:
+    async with client.stream('GET', url, headers=headers, auth=auth, timeout=None, cookies=cj) as r:
         with open(file_path, "ab") as f:
             time_start_realtime = time_start = time.time()
 
@@ -460,15 +510,22 @@ async def _download_data(client, url, folder=None, file_name=None, retry=0):
             return True
 
 
-async def creat_tasks(urls, folder, file_names, limit, desc, retry):
+async def creat_tasks(urls, folder, authorize_from_browser, file_names, limit, desc, allow_redirects, retry):
     limits = httpx.Limits(max_keepalive_connections=limit,
                           max_connections=limit)
     async with httpx.AsyncClient(limits=limits, timeout=None, verify=False) as client:
         if file_names:
-            tasks = [asyncio.ensure_future(_download_data(client, url, folder, file_names[i], retry))
+            tasks = [asyncio.ensure_future(_download_data(client, url, folder,
+                                                          authorize_from_browser=authorize_from_browser,
+                                                          file_name=file_names[i],
+                                                          allow_redirects=allow_redirects,
+                                                          retry=retry))
                      for i, url in enumerate(urls)]
         else:
-            tasks = [asyncio.ensure_future(_download_data(client, url, folder, retry))
+            tasks = [asyncio.ensure_future(_download_data(client, url, folder,
+                                                          authorize_from_browser=authorize_from_browser,
+                                                          allow_redirects=allow_redirects,
+                                                          retry=retry))
                      for url in urls]
 
         # Total process bar
@@ -480,7 +537,8 @@ async def creat_tasks(urls, folder, file_names, limit, desc, retry):
             await coroutine
 
 
-def async_download_datas(urls, folder=None, file_names=None, limit=30, desc='', retry=0):
+def async_download_datas(urls, folder=None, authorize_from_browser=False,
+                         file_names=None, limit=30, desc='', allow_redirects=False, retry=0):
     '''Download multiple files simultaneously.
 
     Parameters:
@@ -489,6 +547,12 @@ def async_download_datas(urls, folder=None, file_names=None, limit=30, desc='',
         iterator contains urls
     folder: str
         the folder to store output files. Default current folder.
+    authorize_from_browser: bool
+        Whether to load cookies used by your web browser for authorization.
+        This means you can use python to download data by logining in to website 
+        via browser (So far the following browsers are supported: Chrome，Firefox, 
+        Opera, Edge, Chromium"). It will be very usefull when website doesn't support
+        "HTTP Basic Auth". Default is False.
     file_names: iterator
         iterator contains names of files. Leaving it None if you want the program
         to parse them from website. file_names can cantain the absolute paths if folder is None.
@@ -496,6 +560,10 @@ def async_download_datas(urls, folder=None, file_names=None, limit=30, desc='',
         the number of files downloading simultaneously
     desc: str
         description of datas downloading
+    allow_redirects: bool
+        Enables or disables HTTP redirects
+    retry: int
+        number of reconnections when status code is 503
 
     Example:
     ---------
@@ -518,14 +586,15 @@ def async_download_datas(urls, folder=None, file_names=None, limit=30, desc='',
     loop = asyncio.SelectorEventLoop(selector)
     try:
         loop.run_until_complete(creat_tasks(
-            urls, folder, file_names, limit, desc, retry))
+            urls, folder, authorize_from_browser, file_names, limit, desc, allow_redirects, retry))
     finally:
         loop.close()
 
 
-async def _is_response_staus_ok(client, url, timeout):
+async def _is_response_staus_ok(client, url, authorize_from_browser, timeout):
+    cj = _get_cookiejar(authorize_from_browser)
     try:
-        r = await client.head(url, timeout=timeout)
+        r = await client.head(url, timeout=timeout, cookies=cj)
         r.close()
         if r.status_code == httpx.codes.OK:
             return True
@@ -535,18 +604,18 @@ async def _is_response_staus_ok(client, url, timeout):
         return False
 
 
-async def creat_tasks_status_ok(urls, limit, timeout):
+async def creat_tasks_status_ok(urls, limit, authorize_from_browser, timeout):
     limits = httpx.Limits(max_keepalive_connections=limit,
                           max_connections=limit)
     async with httpx.AsyncClient(limits=limits, timeout=None) as client:
-        tasks = [asyncio.create_task(_is_response_staus_ok(client, url, timeout))
+        tasks = [asyncio.create_task(_is_response_staus_ok(client, url, authorize_from_browser, timeout))
                  for url in urls]
         status_ok = await asyncio.gather(*tasks)
 
     return status_ok
 
 
-def status_ok(urls, limit=200, timeout=60):
+def status_ok(urls, limit=200, authorize_from_browser=False, timeout=60):
     '''Simultaneously detecting whether the given links are accessable.
 
     Parameters
@@ -555,6 +624,12 @@ def status_ok(urls, limit=200, timeout=60):
         iterator contains urls
     limit: int
         the number of urls connecting simultaneously
+    authorize_from_browser: bool
+        Whether to load cookies used by your web browser for authorization.
+        This means you can use python to download data by logining in to website 
+        via browser (So far the following browsers are supported: Chrome，Firefox, 
+        Opera, Edge, Chromium"). It will be very usefull when website doesn't support
+        "HTTP Basic Auth". Default is False.
     timeout: int
         Request to stop waiting for a response after a given number of seconds
 
@@ -584,7 +659,7 @@ def status_ok(urls, limit=200, timeout=60):
     loop = asyncio.SelectorEventLoop(selector)
     try:
         status_ok = loop.run_until_complete(
-            creat_tasks_status_ok(urls, limit, timeout))
+            creat_tasks_status_ok(urls, limit, authorize_from_browser, timeout))
     # Zero-sleep to allow underlying connections to close
     finally:
         loop.close()
